/*
 * Color/Palette funcs
 */

#include <stdlib.h>
#include <stdio.h>
#include <vgagl.h>

#include "hr_defines.h"
#include "hr_general.h"

struct hr_color{
	int colors;
	unsigned char *index;
};

struct hr_palette{
	int flags;
	unsigned char *color_flags;
	unsigned char *palette_store;
};


/* Returns color index that is stored in position "i" in color
 * struct "color".*/
int hr_get_color_index(struct hr_color *color,int i)
{
	int c=0;
	if(color!=NULL){
		if(i<(color->colors))
		  c=(color->index[i]);
		else
		  c=(color->index[color->colors-1]);
	}
	return c;
}


/* Re-sets an index in a color struct.*/
void hr_set_color_index(struct hr_color *color,int i,int newindex)
{
	if(color!=NULL){
		if(i<(color->colors))
		  color->index[i]=newindex;
		else
		  color->index[color->colors-1]=newindex;
	}
}


/* Sets flags of given color within a palette.
 * Returns 0 if an error occurs.*/
int hr_set_color(struct hr_palette *palette,int color,int flags)
{
	if(palette!=NULL){
		*((palette->color_flags)+color)=(unsigned char)flags;
		return 1;
	}
	else
	  return 0;
}


/* Returns index of first unused color in a palette*/
int hr_first_free_color(struct hr_palette *palette)
{
	int l;
	if(palette!=NULL){
		for(l=1;l<256;l++){
			if(!(*((palette->color_flags)+l) & COLOR_USED))
			  return l;
		}
	}
	return 0;
}


/* Rotates colors on given range.*/
void hr_rotate_colors(int mincol,int maxcol,int step)
{
#if 0
	int r,g,b;
	if((mincol>=0)&&(mincol<256)&&(maxcol>=0)&&(maxcol<256)){
		/* FIX */
	}
#endif
}



/* Stores currect screen palette to a table.*/
void hr_store_palette(struct hr_palette *palette)
{
	if(palette!=NULL){
		if(palette->palette_store==NULL)
		  palette->palette_store=malloc(256*3);
		gl_getpalettecolors(0,256,palette->palette_store);
		palette->flags |= COLOR_PALETTE_STORED;
	}
}


/* Deletes previously stored color table in a palette.*/
void hr_free_palette(struct hr_palette *palette)
{
	if(palette!=NULL){
		palette->flags&=~(COLOR_PALETTE_STORED);
		free(palette->palette_store);
	}
}


/* Sets screen colors to ones stored in a palette */
void hr_use_palette(struct hr_palette *palette)
{
	if(palette!=NULL){
		hr.active_palette=palette;
		if((palette->flags & COLOR_PALETTE_STORED)&&
			(palette->palette_store!=NULL))
		  gl_setpalettecolors(0,256,palette->palette_store);
	}
}


/* Creates a palette.*/
struct hr_palette *hr_create_palette(int flags)
{
	struct hr_palette *palette;
	palette=malloc(sizeof(struct hr_palette));
	palette->flags=flags;
	palette->color_flags=malloc(256*sizeof(unsigned char));
	palette->palette_store=NULL;
	return palette;
}


/* Deletes a palette.*/
struct hr_palette *hr_delete_palette(struct hr_palette *palette)
{
	if(palette!=NULL){
		free(palette->color_flags);
		free(palette->palette_store);
		free(palette);
	}
	return palette;
}


/* Creates a color struct.*/
struct hr_color *hr_create_color_struct(int colors)
{
   struct hr_color *ss;
	int c;
	if((ss=malloc(sizeof(struct hr_color)))){
		if((ss->index=malloc(sizeof(unsigned char)*colors))){
			ss->colors=colors;
			/* Make a copy of the default color struct if one exists.*/
			if(hr.def_colors!=NULL){
				for(c=0;c<MIN((hr.def_colors->colors),colors);c++)
				  ss->index[c]=(hr.def_colors->index[c]);
				/* If there were more colors initialized, init them.*/
				if(colors>(hr.def_colors->colors)){
					for(c=(hr.def_colors->colors);c<colors;c++)
					  ss->index[c]=COLOR_DEFAULT;
				}
			}
			/* Fill index table with COLOR_DEFAULT value.*/
			else{
				for(c=0;c<colors;c++)
				  ss->index[c]=COLOR_DEFAULT;
			}
		}
		else{
			free(ss);
			return NULL;
		}
	}
	return ss;
}


/* Deletes a color struct.*/
struct hr_color *hr_delete_color_struct(struct hr_color *colors)
{
	if(colors!=NULL){
		free(colors->index);
		free(colors);
		return colors;
	}
	return NULL;
}
